@@ -110,8 +110,8 @@ def handle_additional_edges(project, statements):
             variable_ids.add(variable)
             variable_dict=entity_dict.get(variable, None)
             if variable_dict is not None:
-                label=variable_dict.get("label", "A "+variable)
-                description=variable_dict.get("description", variable+" relation")
+                label=variable_dict.get("label", variable)
+                description=variable_dict.get("description", "")
                 data_type=variable_dict.get("data_type", "quantity")
                 if data_type.lower()=="wikibaseitem":
                     qnode_ids.add(statement["value"])
@@ -140,9 +140,8 @@ def handle_additional_edges(project, statements):
     
     for qnode_id in qnode_ids:
         variable_dict=entity_dict.get(qnode_id, {})
-        label=variable_dict.get("label", "A "+qnode_id)
-        #description=variable_dict.get("description", "A "+variable)
-        if variable_dict is not None:
+        if variable_dict:
+            label=variable_dict.get("label", qnode_id)
             tsv_data+=create_metadata_for_custom_qnode(qnode_id, label)
 
     for result_dict in tsv_data:
