@@ -262,16 +262,16 @@ class Project:
             raise ValueError("The new name you have provided already exists in the project directory")
             
         if old_name in self.data_files: #handle data files completely separately from everything else
-            is_csv=False
             old_csv_name=Path(old_name).stem
             new_csv_name=Path(new_name).stem
+            uses_csv_sheet_name=False
             if len(self.data_files[old_name]["val_arr"])==1 and self.data_files[old_name]["selected"]==old_csv_name: #it's a csv
-                is_csv=True
+                uses_csv_sheet_name=True
             for edit_dict in [self.annotations, self.data_files, self.yaml_sheet_associations]:
                 for key in edit_dict:
                     if key==old_name:
                         edit_dict[new_name]=edit_dict.pop(old_name)
-                        if is_csv:
+                        if uses_csv_sheet_name:
                             try:
                                 edit_dict[new_name][new_csv_name]=edit_dict[new_name].pop(old_csv_name)
                             except KeyError: #.data_files
